﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Configuration;
using at.uba.uwwtd2008.Items;
using at.uba.uwwtd2008.components;

namespace at.uba.uwwtd2008.DataAccess {
    public enum DataConnectionType {
        undefined,
        oledb,
        odbc
    }

    public class DataAccessFactory {
        private static DataAccessFactory instance = null;
        private DataSet dsReporters = null;

        private DataConnectionType mType = DataConnectionType.undefined;
        private string mConnectionString = null;

        private DataAccessFactory() {
            Configuration conf = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            ConnectionStringSettings str = conf.ConnectionStrings.ConnectionStrings["at.uba.uwwtd2008.Properties.Settings.Uwwtd2009ConnectionString"];
            if (str != null)
            {
                if (str.ProviderName.Equals("System.Data.OleDb"))
                    mType = DataConnectionType.oledb;
                if (str.ProviderName.Equals("System.Data.Odbc"))
                    mType = DataConnectionType.odbc;

                mConnectionString = str.ConnectionString;
            }
            else
            {

                mType = DataConnectionType.oledb;
                mConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=.\\Uwwtd2009.mdb";
            }

            initReporters();
        }

        private static System.Globalization.CultureInfo cultureInfo = System.Globalization.CultureInfo.GetCultureInfo("en-US");

        public static object parseDBDecimal(object objValue)
        {
            if (objValue == null) return null;
            if (objValue is DBNull) return null;
            if (objValue is decimal) return (decimal)objValue;
            if (objValue is string) return decimal.Parse(objValue.ToString(), cultureInfo.NumberFormat);
            return decimal.Parse(objValue.ToString());
        }

        public static string toDBDecimal(object value)
        {
            if (value == null) return null;
            if (value is DBNull) return null;
            
            return ((decimal) value).ToString(cultureInfo.NumberFormat);
        }


        public void initReporters()
        {
            try
            {
                IDbConnection conn = GetConnection();
                try
                {
                    dsReporters = new DataSet();

                    IDbDataAdapter reporterTAdapter = CreateDataAdapter(CreateCommand("Select * from T_Reporter", conn));
                    IDbDataAdapter repperiodTAdapter = CreateFullDataAdapter(CreateCommand("select * from T_ReportPeriod", conn));
                    IDbDataAdapter contactTAdapter = CreateFullDataAdapter(CreateCommand("select * from T_Contact", conn));
                    IDbDataAdapter lovTAdapter = CreateFullDataAdapter(CreateCommand("select * from T_LOV where lovName='MS'", conn));

                    FillDataAdapter(reporterTAdapter, dsReporters, "T_Reporter");
                    FillDataAdapter(repperiodTAdapter, dsReporters, "T_ReportPeriod");
                    FillDataAdapter(contactTAdapter, dsReporters, "T_Contact");
                    FillDataAdapter(lovTAdapter, dsReporters, "T_Lov");

                    // Add Relations
                    if (!dsReporters.Relations.Contains("T_ReporterT_ReportPeriod"))
                        dsReporters.Relations.Add("T_ReporterT_ReportPeriod", dsReporters.Tables["T_Reporter"].Columns["rptID"], dsReporters.Tables["T_ReportPeriod"].Columns["rptID"]);
                    if (!dsReporters.Relations.Contains("T_ReportPeriodT_Contact"))
                        dsReporters.Relations.Add("T_ReportPeriodT_Contact", dsReporters.Tables["T_ReportPeriod"].Columns["repID"], dsReporters.Tables["T_Contact"].Columns["repID"]);
                    if (!dsReporters.Relations.Contains("T_Lov_T_Reporter"))
                        dsReporters.Relations.Add("T_Lov_T_Reporter", dsReporters.Tables["T_Lov"].Columns["lovID"], dsReporters.Tables["T_Reporter"].Columns["rptMState"]);
                }
                finally
                {
                    conn.Close();
                }
            }
            catch (Exception)
            {
                // do nothing in here - just prevent the system from crashing
            }

        }

        public static DataAccessFactory Instance {
            get {
                if (instance == null)
                {
                    instance = new DataAccessFactory();
                }
                return instance;
            }
        }

        public IDbConnection GetConnection() {
            IDbConnection conn = null;
            switch (mType)
            {
                case DataConnectionType.oledb:
                    conn = new System.Data.OleDb.OleDbConnection(mConnectionString);
                    break;
                case DataConnectionType.odbc:
                    conn = new System.Data.Odbc.OdbcConnection(mConnectionString);
                    break;
                default:
                    throw new ArgumentException("Connection Type is not set!");
            }

            conn.Open();
            
            return conn;
        }

        public IDbCommand CreateCommand(string strCommand, IDbConnection conn) {
            IDbCommand cmd = CreateCommand(strCommand, conn, null);
            //switch (mType)
            //{
            //    case DataConnectionType.oledb:
            //        cmd = new System.Data.OleDb.OleDbCommand(strCommand, (System.Data.OleDb.OleDbConnection) conn);
            //        break;
            //    case DataConnectionType.odbc:
            //        cmd = new System.Data.Odbc.OdbcCommand(strCommand, (System.Data.Odbc.OdbcConnection)conn);
            //        break;
            //    default:
            //        throw new ArgumentException("Connection Type is not set!");
            //}
            return cmd;
        }

        public IDbCommand CreateCommand(string strCommand, IDbConnection conn, IDbTransaction trans) {
            IDbCommand cmd = null;
            switch (mType)
            {
                case DataConnectionType.oledb:
                    cmd = new System.Data.OleDb.OleDbCommand(strCommand, (System.Data.OleDb.OleDbConnection)conn, (System.Data.OleDb.OleDbTransaction) trans);
                    break;
                case DataConnectionType.odbc:
                    cmd = new System.Data.Odbc.OdbcCommand(strCommand, (System.Data.Odbc.OdbcConnection)conn, (System.Data.Odbc.OdbcTransaction) trans);
                    break;
                default:
                    throw new ArgumentException("Connection Type is not set!");
            }
            return cmd;
        }

        public IDbDataParameter CreateParameter(string strName, DbType type, int size, string column, object value)
        {
            IDbDataParameter param = null;
            switch (mType)
            {
                case DataConnectionType.oledb:
                    param = new System.Data.OleDb.OleDbParameter(strName, DBNull.Value);
                    param.DbType = type;
                    param.SourceColumn = column;
                    param.Size = size;
                    if (value == null) value = DBNull.Value;
                    param.Value = value;
                    break;
                case DataConnectionType.odbc:
                    param = new System.Data.Odbc.OdbcParameter(strName, DBNull.Value);
                    param.DbType = type;
                    param.SourceColumn = column;
                    param.Size = size;
                    if (value == null) value = DBNull.Value;
                    param.Value = value;
                    break;
                default:
                    throw new ArgumentException("Connection Type is not set!");
            }
            return param;
        }

        public System.Data.Common.DbCommandBuilder CreateCommandBuilder(IDbDataAdapter dataAdapter) {
            switch (mType)
            {
                case DataConnectionType.oledb:
                    return new System.Data.OleDb.OleDbCommandBuilder((System.Data.OleDb.OleDbDataAdapter) dataAdapter);
                case DataConnectionType.odbc:
                    return new System.Data.Odbc.OdbcCommandBuilder((System.Data.Odbc.OdbcDataAdapter) dataAdapter);
                default:
                    throw new ArgumentException("Connection Type is not set!");
            }
        }

        public IDbDataAdapter CreateDataAdapter(IDbCommand cmd) {
            IDbDataAdapter adapter = null;
            switch (mType)
            {
                case DataConnectionType.oledb:
                    adapter = new System.Data.OleDb.OleDbDataAdapter((System.Data.OleDb.OleDbCommand) cmd);
                    break;
                case DataConnectionType.odbc:
                    adapter = new System.Data.Odbc.OdbcDataAdapter((System.Data.Odbc.OdbcCommand)cmd);
                    break;
                default:
                    throw new ArgumentException("Connection Type is not set!");
            }
            adapter.MissingSchemaAction = MissingSchemaAction.AddWithKey;
            adapter.MissingMappingAction = MissingMappingAction.Passthrough;
            return adapter;
        }

        public IDbDataAdapter CreateFullDataAdapter(IDbCommand cmd) {
            IDbDataAdapter adapter = CreateDataAdapter(cmd);

            System.Data.Common.DbCommandBuilder builder = CreateCommandBuilder(adapter);
            builder.SetAllValues = true;
            adapter.InsertCommand = builder.GetInsertCommand(true);
            adapter.InsertCommand.Connection = cmd.Connection;
            adapter.InsertCommand.Transaction = cmd.Transaction;
            adapter.UpdateCommand = builder.GetUpdateCommand(true);
            adapter.UpdateCommand.Connection = cmd.Connection;
            adapter.UpdateCommand.Transaction = cmd.Transaction;
            adapter.DeleteCommand = builder.GetDeleteCommand(true);
            adapter.DeleteCommand.Connection = cmd.Connection;
            adapter.DeleteCommand.Transaction = cmd.Transaction;

            //// refine update Statement for things with decimal-Clumns
            //foreach (IDataParameter param in adapter.UpdateCommand.Parameters)
            //{
            //    if (param.DbType.Equals(DbType.Decimal) || param.DbType.Equals(typeof(System.Decimal)))
            //    {
            //        if (param is System.Data.OleDb.OleDbParameter)
            //        {
            //            ((System.Data.OleDb.OleDbParameter)param).Scale = 6;
            //            ((System.Data.OleDb.OleDbParameter)param).Precision = 18;
            //        }
            //    }
            //}
            return adapter;
        }

        public void FillDataAdapter(IDbDataAdapter adapter, DataSet ds, string tableName) {
            bool constraintsForced = ds.EnforceConstraints;
            ds.EnforceConstraints = false;
            try
            {
                if (ds.Tables.Contains(tableName))
                {
                    ds.Tables[tableName].Clear();
                }

                if (adapter is System.Data.OleDb.OleDbDataAdapter)
                {
                    ((System.Data.OleDb.OleDbDataAdapter)adapter).Fill(ds, tableName);
                }
                else if (adapter is System.Data.Odbc.OdbcDataAdapter)
                {
                    ((System.Data.Odbc.OdbcDataAdapter)adapter).Fill(ds, tableName);
                }
            }
            finally
            {
                ds.EnforceConstraints = constraintsForced;
            }
            
        }

        public void UpdateDataAdapter(IDbDataAdapter adapter, DataSet ds, string tableName) {
            try
            {
                if (adapter is System.Data.OleDb.OleDbDataAdapter)
                {
                    ((System.Data.OleDb.OleDbDataAdapter)adapter).Update(ds, tableName);
                    ds.Tables[tableName].AcceptChanges();
                }
                else if (adapter is System.Data.Odbc.OdbcDataAdapter)
                {
                    ((System.Data.Odbc.OdbcDataAdapter)adapter).Update(ds, tableName);
                    ds.Tables[tableName].AcceptChanges();
                }
            }
            catch (DBConcurrencyException ex)
            {
                System.Diagnostics.EventLog.WriteEntry("UWWTD2008", "Exception while saving: " + ex.Message, System.Diagnostics.EventLogEntryType.Warning, 0, 0, Encoding.ASCII.GetBytes(ex.StackTrace));
            }
        }

        public void UpdateDataAdapter(IDbDataAdapter adapter, DataSet ds, string tableName, DataViewRowState rowState)
        {
            if (adapter is System.Data.OleDb.OleDbDataAdapter)
            {
                ((System.Data.OleDb.OleDbDataAdapter)adapter).Update(ds.Tables[tableName].Select("","",rowState));
            }
            else if (adapter is System.Data.Odbc.OdbcDataAdapter)
            {
                ((System.Data.Odbc.OdbcDataAdapter)adapter).Update(ds.Tables[tableName].Select("", "", rowState));
            }
        }

        #region reporter data
        public DataSet getReporterDataSet()
        {
            return dsReporters;
        }

        public object getRepIDforReportCode(string repCode)
        {
            DataRow[] rows = dsReporters.Tables["T_ReportPeriod"].Select("repCode = '" + repCode + "'");
            if (rows.Length < 1) return null;
            return rows[0]["repID"];
        }

        public DataRow getRepPeridodforRepID(object repID) {
            return dsReporters.Tables["T_ReportPeriod"].Rows.Find(repID);
        }
        
        #endregion

        #region Common Data
        private ReportPeriod currentReportPeriod = null;
        private DataSet commonDataSet = new DataSet();
        public const string TNAME_NUTSREP = "nutsrep";
        public const string TNAME_NUTS = "nuts";
        public const string TNAME_LOV  = "lov";
        public const string TNAME_DATAFIELDS = "datafields";

        public const string TNAME_BIGCITIES = "bigcities";


        public void setCurrentReportPeriod(ReportPeriod period) {
            if ((period == null) || (currentReportPeriod == null) || (currentReportPeriod.RepID != period.RepID)) {
                currentReportPeriod = period;
                commonDataSet = new DataSet();
            } // if
        }

        public DataTable getBigCitiesForCurrentReporter()
        {
            if (commonDataSet == null) throw new ArgumentException("The current Reporter cannot be used. Please select a reporter first");
            if (!commonDataSet.Tables.Contains(TNAME_BIGCITIES))
            {
                IDbConnection conn = GetConnection();
                try
                {
                    FillDataAdapter(
                        CreateDataAdapter(
                            CreateCommand("Select bigID, bigCity from T_BigCity WHERE bigCountryCode = '" + currentReportPeriod.Reporter.State.Key + "'", conn)),
                            commonDataSet, TNAME_BIGCITIES);

                    // this here is only allowed because we will NEVER put this back to the DB
                    commonDataSet.Tables[TNAME_BIGCITIES].Columns["bigID"].AllowDBNull = true;
                    DataRow row = commonDataSet.Tables[TNAME_BIGCITIES].NewRow();
                    row["bigCity"] = "<none>";
                    row["bigID"] = System.DBNull.Value;
                    commonDataSet.Tables[TNAME_BIGCITIES].Rows.InsertAt(row, 0);
                    commonDataSet.Tables[TNAME_BIGCITIES].AcceptChanges();
                }
                finally
                {
                    conn.Close();
                }
            }

            return commonDataSet.Tables[TNAME_BIGCITIES];
        }

        public DataTable getNUTSTableForCurrentReporter() {
            if (commonDataSet == null) throw new ArgumentException("The current Reporter cannot be used. Please select a reporter first");
            if (!commonDataSet.Tables.Contains(TNAME_NUTSREP))
            {
                IDbConnection conn = GetConnection();
                try
                {
                    FillDataAdapter(
                        CreateDataAdapter(
                            CreateCommand("select nutID, nutNUTS, nutRegion from T_NUTS where nutCountryCode = '" + currentReportPeriod.Reporter.State.Key + "'", conn)),
                            commonDataSet, TNAME_NUTSREP);
                }
                finally
                {
                    conn.Close();
                }
            }

            return commonDataSet.Tables[TNAME_NUTSREP];
        }

        public DataTable getNutsTable()
        {
            if (!commonDataSet.Tables.Contains(TNAME_NUTS))
            {
                IDbConnection conn = GetConnection();
                try
                {
                    FillDataAdapter(
                        CreateDataAdapter(
                            CreateCommand("select * from T_NUTS", conn)),
                            commonDataSet, TNAME_NUTS);
                }
                finally
                {
                    conn.Close();
                }
            }

            return commonDataSet.Tables[TNAME_NUTS];
        }

        public object getNUTSIdForNUTSCode(string nutsCode)
        {
            DataRow[] rows = getNutsTable().Select("nutNUTS = \'" + nutsCode + "\'");
            if (rows.Length < 1) return DBNull.Value;
            return rows[0]["nutID"];
        }

        public string getNutsCode(object nutsID)
        {
            if ((nutsID == null) || (nutsID is DBNull)) return "";
            return getNutsTable().Rows.Find(nutsID)["nutNUTS"] as string;
        }

        public DataTable getLOVTable() {
            if (commonDataSet == null) throw new ArgumentException("The current Reporter cannot be used. Please select a reporter first");
            if (!commonDataSet.Tables.Contains(TNAME_LOV))
            {
                IDbConnection conn = GetConnection();
                try
                {
                    FillDataAdapter(
                        CreateDataAdapter(
                            CreateCommand("select * from T_LOV", conn)),
                            commonDataSet, TNAME_LOV);
                    commonDataSet.Tables[TNAME_LOV].Columns[0].AllowDBNull = true;
                    commonDataSet.Tables[TNAME_LOV].Columns[0].AutoIncrement = false;
                }
                finally
                {
                    conn.Close();
                }
            }

            return commonDataSet.Tables[TNAME_LOV];
        }

        public DataTable getDATAFieldsTable()
        {
            if (commonDataSet == null) throw new ArgumentException("The current Reporter cannot be used. Please select a reporter first");
            if (!commonDataSet.Tables.Contains(TNAME_DATAFIELDS))
            {
                IDbConnection conn = GetConnection();
                try
                {
                    FillDataAdapter(
                        CreateDataAdapter(
                            CreateCommand("select * from T_DATAFIELDS", conn)),
                            commonDataSet, TNAME_DATAFIELDS);
                }
                finally
                {
                    conn.Close();
                }
            }

            return commonDataSet.Tables[TNAME_DATAFIELDS];
        }

        public DataTable getYNLovValues()
        {
            return getLovValuesByKey("YN");
        }

        public DataTable getCOLLDETLovValues()
        {
            return getLovValuesByKey("COLLDET");
        }

        public DataTable getPASSLovValues()
        {
            return getLovValuesByKey("PASS");
        }

        public DataTable getRATYPLovValues()
        {
            return getLovValuesByKey("RATYP");
        }

        public DataTable getLOADDETValues()
        {
            return getLovValuesByKey("LOADDET");
        }

        public DataTable getUWWTPCONNValues() 
        {
            return getLovValuesByKey("UWWTP");
        }

        public DataTable getRATYPValues()
        {
            DataTable table = getLovValuesByKey("RATYP");
            //table.Columns["lovID"].Unique = true;
            //table.Columns["lovID"].AllowDBNull = false;
            //table.PrimaryKey = new DataColumn[] {table.Columns["lovID"]};
            return table;
        }

        public DataTable getCOLLDETLovValuesNullable()
        {
            DataTable table = getLovValuesByKey("COLLDET");
            table.Columns["lovID"].AllowDBNull = true;
            DataRow newRow = table.NewRow();
            newRow["lovID"] = DBNull.Value;
            newRow["lovName"] = "COLLDET";
            newRow["lovWert"] = "<none>";
            table.Rows.InsertAt(newRow, 0);

            return table;
        }

        public DataTable getLovValuesByKey(string key)
        {
            DataTable dtLov = getLOVTable();
            DataRow[] rows = dtLov.Select("lovName = '"+key+"'", "lovID DESC");
            DataTable dt = new DataTable();
            foreach (DataColumn col in dtLov.Columns)
            {
                dt.Columns.Add(col.ColumnName, col.DataType, col.Expression);
            }
            foreach (DataRow row in rows)
            {
                dt.Rows.Add(row.ItemArray);
            }

            return dt;
        }

        public DataView getLovViewByKey(string name)
        {
            DataView dv = new DataView(getLOVTable());
            dv.RowFilter = "lovName = '" + name + "'";
            return dv;
        }

        public DataRow getLovRowByID(object lovID)
        {
            DataRow row = getLOVTable().Rows.Find(lovID);
            if (row == null)
            {
                row = getLOVTable().NewRow();
            }
            return row;
        }

        public DataRow getLovRowByKey(string name, string key)
        {
            if (key == null) return getLOVTable().NewRow();
            
            DataView dv = new DataView(getLOVTable());
            dv.RowFilter = "lovName = '" + name + "' and lovKey = '"+key+"'";
            if (dv.Count < 1) return getLOVTable().NewRow();
            if (dv.Count > 1) throw new InvalidOperationException("Query returned more than one row");
            return dv[0].Row;
        }

        public DataView getLovSTAT()
        {
            return getLovViewByKey("STAT");
        }
        #endregion

        #region RA-Helpers
        public static int oldRATypToNew(int oldRAType)
        {
            switch (oldRAType) {
                case 1: // nothing
                    return CtrlReceivingAreasEdit.RATYP_NORMAL;
                case 2: // Art 58/23
                    return CtrlReceivingAreasEdit.RATYP_ART5253;
                case 3: // Art 58/54
                    return CtrlReceivingAreasEdit.RATYP_ART54;
                case 4: // Sensitive Area
                    return CtrlReceivingAreasEdit.RATYP_CSASALSA | CtrlReceivingAreasEdit.RATYP_ART51;
                case 5: // Less Sensitive Area
                    return CtrlReceivingAreasEdit.RATYP_CSASALSA | CtrlReceivingAreasEdit.RATYP_ART61;
                case 6: // LSA + SA
                    return CtrlReceivingAreasEdit.RATYP_CSASALSA | CtrlReceivingAreasEdit.RATYP_ART51 | CtrlReceivingAreasEdit.RATYP_ART61;
                default: // basis
                    return 0;
            }
        }

        public static int newRATypToOld(int newRAType) {
            switch (newRAType)
            {
                case CtrlReceivingAreasEdit.RATYP_NORMAL: // nothing
                    return 1;
                case CtrlReceivingAreasEdit.RATYP_ART5253: // Art 58/23
                    return 2;
                case CtrlReceivingAreasEdit.RATYP_ART54: // Art 58/54
                    return 3;
                case CtrlReceivingAreasEdit.RATYP_CSASALSA | CtrlReceivingAreasEdit.RATYP_ART51: // Sensitive Area
                    return 4;
                case CtrlReceivingAreasEdit.RATYP_CSASALSA | CtrlReceivingAreasEdit.RATYP_ART61: // Less Sensitive Area
                    return 5;
                case CtrlReceivingAreasEdit.RATYP_CSASALSA | CtrlReceivingAreasEdit.RATYP_ART51 | CtrlReceivingAreasEdit.RATYP_ART61: // LSA + SA
                    return 6;
                default: // basis
                    return 0;
            }
        }
        #endregion

        #region DataConversion Routines
        public static float getFloatValue(object obj)
        {
            if (obj == null) return 0f;
            if (obj is DBNull) return 0f;
            if (obj is float) return (float)obj;
            try
            {
                string strValue = obj.ToString();
                if (strValue.Length < 1) return 0f;
                return float.Parse(strValue);
            }
            catch (FormatException)
            {
                return float.MinValue;
            }
        }
        public static double getDoubleValue(object obj)
        {
            if (obj == null) return 0d;
            if (obj is DBNull) return 0d;
            if (obj is double) return (double)obj;
            try
            {
                string strValue = obj.ToString();
                if (strValue.Length < 1) return 0d;
                return double.Parse(strValue);
            }
            catch (FormatException)
            {
                return double.MinValue;
            }
        }
        public static long getLongValue(object obj)
        {
            if (obj == null) return 0;
            if (obj is DBNull) return 0;
            if (obj is long) return (long)obj;
            try
            {
                string strValue = obj.ToString();
                if (strValue.Length < 1) return 0;
                return long.Parse(strValue);
            }
            catch (FormatException)
            {
                return long.MinValue;
            }
        }
        public static int getIntValue(object obj)
        {
            if (obj == null) return 0;
            if (obj is DBNull) return 0;
            if (obj is int) return (int)obj;
            try
            {
                string strValue = obj.ToString();
                if (strValue.Length < 1) return 0;
                return int.Parse(strValue);
            }
            catch (FormatException)
            {
                return int.MinValue;
            }
        }
        #endregion
    }
}
