﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using at.uba.uwwtd2008.Items;
using at.uba.uwwtd2008.xml;


namespace at.uba.uwwtd2008.DataAccess
{
    abstract class InterfaceBase
    {
        protected IDbConnection conn = null;
        protected IDbTransaction trans = null;
        protected DataFormatConverter conv = null;
        protected DataAccessFactory dataFactory = DataAccessFactory.Instance;
        protected LoggingClass logger;
        protected string logPrefix;

        public InterfaceBase(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv)
        {
            this.conn = conn;
            this.trans = trans;
            this.conv = conv;
            logPrefix = this.GetType().Name.Substring(9);
        }

        public LoggingClass Logger
        {
            set { logger = value; }
            get { return logger; }
        }

        public void reportFail()
        {
            logger.reportFail(logPrefix);
        }

        public void reportSuccess()
        {
            logger.reportSuccess(logPrefix);
        }

        //protected bool checkMandatory(string table, string field, object item)
        //{
        //    DataTable tab = dataFactory.getDATAFieldsTable();
        //    DataRow []rows = tab.Select("dfTableName = '"+table+"' AND dfFieldName = '"+field+"'");
        //    if (rows.Length < 1) return true;
        //    if (rows[0]["dfMandatory"] == null) return true;
        //    if (rows[0]["dfMandatory"] is DBNull) return true;
        //    if (!rows[0]["dfMandatory"].Equals("Y")) return true;
        //    return (item != null) && !(item is DBNull);
        //}

        protected string getReaderValueString(object value)
        {
            if ((value is DBNull) || (value == null)) return "";
            if (value is string) return value as string;
            if (value is DateTime) return conv.formatDateLong((DateTime) value);
            if (value is double) return conv.format((double) value);
            if (value is float) return conv.format((float) value);
            if (value is decimal) return conv.format((decimal) value);
            if (value is long) return conv.format((long) value);
            if (value is int) return conv.format((int) value);
            if (value is bool) return ((bool) value) ? "Y" : "N";
            return value.ToString();
        }

        protected void setParameterValue(IDbCommand cmd, string name, object value)
        {
            setParameterValue(cmd, cmd.Parameters.IndexOf(name), value);
        }

        protected void setParameterValue(IDbCommand cmd, int idx, object value)
        {
            IDbDataParameter param = (IDbDataParameter)cmd.Parameters[idx];
            string strVal = null;

            if (value == null)
            {
                param.Value = DBNull.Value;
                return;
            }

            if (value is DBNull)
            {
                param.Value = value;
                return;
            }

            // Workaround to get around the DB-Driver limitation with decimal values
            if (param.SourceColumn.EndsWith("Latitude") ||
                param.SourceColumn.EndsWith("Longitude"))
            {
                if (value is decimal)
                {
                    param.Value = DataAccessFactory.toDBDecimal((decimal)value);
                    return;
                }

                strVal = value as string;
                if (strVal == null)
                {
                    param.Value = DBNull.Value;
                    return;
                }

                object val = conv.importDecimal(strVal);
                if (val is decimal)
                    param.Value = DataAccessFactory.toDBDecimal((decimal)val);
                else if (val is DBNull)
                    param.Value = val;
                else
                    param.Value = DBNull.Value;

                return;
            }

            strVal = value as string;
            if (strVal == null)
            {
                param.Value = value;
                return;
            }

            if (param.DbType == DbType.Int32)
            {
                param.Value = conv.importInt32(strVal);
            }
            else if (param.DbType == DbType.Int64)
            {
                param.Value = conv.importInt64(strVal);
            }
            else if (param.DbType == DbType.Decimal)
            {
                param.Value = conv.importDecimal(strVal);
            }
            else if (param.DbType == DbType.Double)
            {
                param.Value = conv.importDouble(strVal);
            }
            else if (param.DbType == DbType.Single)
            {
                param.Value = (float)conv.importDouble(strVal);
            }
            else if ((param.DbType == DbType.Time) || (param.DbType == DbType.Date) || (param.DbType == DbType.DateTime))
            {
                param.Value = conv.parseDate(strVal);
            }
            else if ((param.DbType == DbType.String))
            {
                param.Value = strVal;
            }
            else if ((param.DbType == DbType.Boolean))
            {
                if (strVal.ToUpper().Equals("Y") || strVal.ToUpper().Equals("N"))
                    param.Value = strVal.ToUpper().Equals("Y");
                else
                    param.Value = Boolean.Parse(strVal);
            }
            else
            {
                Console.WriteLine("Found invalid Parameter " + param.ParameterName);
            }
        }

        public static decimal convertDecimal(object obj) {
            if ((obj == null) || (obj is DBNull))
            {
                return decimal.MinValue;
            }

            if (obj is float) return (decimal)((float)obj);
            if (obj is double) return (decimal)((double)obj);
            if (obj is decimal) return (decimal)obj;

            return decimal.Parse(obj.ToString());
        }

        public static long convertInt64(object obj) {
            if ((obj == null) || (obj is DBNull))
            {
                return long.MinValue;
            }

            if (obj is float) return (long)((float)obj);
            if (obj is double) return (long)((double)obj);
            if (obj is decimal) return (long)((decimal)obj);
            if (obj is int) return (long)((int)obj);
            if (obj is decimal) return (long)obj;

            return long.Parse(obj.ToString());
        }

        protected void logInfo(string message)
        {
            Logger.log(LoggingLevel.INFO, logPrefix, message);
        }

        protected void logError(string message)
        {
            Logger.log(LoggingLevel.ERROR, logPrefix, message);
        }

        protected void logSpecial(string logLevel, string message)
        {
            Logger.log(logLevel, logPrefix, message);
        }

        public abstract void importData(StreamReader rdr, DataRow repPeriod);

        public abstract void exportData(StreamWriter wr, DataRow repPeriod);

        public abstract void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod);
        
        public abstract void importXML(object collection, DataRow repPeriod);

        /// <summary>
        /// Function must be overwritten by derived classes
        /// This implements the QA-QC Check-Roles
        /// The function also writes the data to the Log
        /// </summary>
        /// <param name="value">The Value of this Field</param>
        /// <param name="fieldName">The name of the Field</param>
        /// <param name="repPeriod">Contains the Reporting Period Row</param>
        /// <param name="isImport">shows, if this is an import-rulecheck or an export-rulecheck</param>
        /// <param name="lineNo">lineNumber in CSV-Files ... should be -1 for all other types</param>
        /// <returns>true, if check was OK, false if an error was found</returns>
        public virtual bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            return true;
        }


        public bool checkFieldMandatoryPresence(object value, string fieldName, ref string errorText)
        {
            return checkFieldMandatoryPresence(value, fieldName, ref errorText, null);
        }

        public bool checkFieldMandatoryPresence(object value, string fieldName, ref string errorText, string additionalText)
        {
            bool isPresent = true;
            if ((value == null) || (value is DBNull))
                isPresent = false;
            else if ((value is string) && (((string)value).Length < 1))
                isPresent = false;

            if (!isPresent)
            {
                errorText = fieldName + " is Master Data";
                if (additionalText != null)
                    errorText = errorText + " - " + additionalText;
            }

            return isPresent;
        }

        public bool checkOptionalPresence(object value, string fieldName, ref string infoText)
        {
            return checkOptionalPresence(value, fieldName, ref infoText, null);
        }

        public bool checkOptionalPresence(object value, string fieldName, ref string infoText, string additionalText)
        {
            bool isPresent = true;
            if ((value == null) || (value is DBNull))
                isPresent = false;
            else if ((value is string) && (((string)value).Length < 1))
                isPresent = false;

            if (!isPresent)
            {
                infoText = fieldName + " is not set";
                if (additionalText != null)
                    infoText = infoText + " - " + additionalText;

            }

            return isPresent;
        }
        public bool checkLOVValue(object value, string fieldName, string lovName, ref string infoText, bool isImport, string additionalText)
        {
            bool retVal = true;
            string errorText = "";

            if ((value == null) || (value is DBNull))
                return true;
            else if ((value is string) && (((string)value).Length < 1))
                return true;

            if (isImport)
            {
                if (!(value is string))
                {
                    errorText = fieldName + " error in program execution - on import this item should be Text";
                    retVal = false;
                } 
                else if (dataFactory.getLovRowByKey(lovName, value as string)["lovID"] is DBNull)
                {
                    retVal = false;
                    errorText = fieldName + " has incorrect value for '"+lovName+"' (" + value + ")";
                    DataView values = dataFactory.getLovViewByKey(lovName);
                    if (values.Count < 5)
                    {
                        errorText = errorText + "; Allowed: ";
                        bool isFirst = true;
                        foreach (DataRowView row in values)
                        {
                            if (!isFirst) errorText = errorText + ", ";
                            isFirst = false;
                            errorText = errorText + row["lovKey"];
                        }
                    }
                }
            }
            else
            {
                if (!dataFactory.getLovRowByID(value)["lovName"].Equals(lovName))
                {
                    retVal = false;
                    errorText = fieldName + " has incorrect value for '" + lovName + "' (" + value + ")";
                }
            }

            if (!retVal)
            {
                infoText = errorText;
                if (additionalText != null)
                    infoText = infoText + " - " + additionalText;

            }

            return retVal;
        }
    }
}
